use std::io;
use std::fmt;
use hex::FromHexError;

#[derive(Debug)]
pub enum Error {
    IO(io::Error),
    Boxed(Box<dyn std::error::Error>),
    BlockModeError(block_modes::BlockModeError),
    InvalidKeyIvLength(block_modes::InvalidKeyIvLength),
    FromHexError(hex::FromHexError)
}

impl fmt::Display for Error {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        match self {
            Error::IO(e) => {
                write!(f, "IO Error: {}", e)
            },
            Error::Boxed(e) => {
                write!(f, "{}", e)
            },
            Error::BlockModeError(e) => {
                write!(f, "CBC Error: {}", e)
            },
            Error::InvalidKeyIvLength(e) => {
                write!(f, "CBC InvalidKeyIvLength Error: {}", e)
            }
            Error::FromHexError(e) => {
                write!(f, "Hex conversion Error: {}", e)
            }
        }
    }
}

impl From<io::Error> for Error {
    fn from(e: io::Error) -> Error {
        Error::IO(e)
    }
}

impl From<block_modes::BlockModeError> for Error {
    fn from(e: block_modes::BlockModeError) -> Error {
        Error::BlockModeError(e)
    }
}

impl From<block_modes::InvalidKeyIvLength> for Error {
    fn from(e: block_modes::InvalidKeyIvLength) -> Error {
        Error::InvalidKeyIvLength(e)
    }
}

impl From<hex::FromHexError> for Error {
    fn from(e: hex::FromHexError) -> Error { Error::FromHexError(e)}
}

pub(crate) type Result<T> = std::result::Result<T, Error>;
